# -*- coding: utf-8 -*-
#!/usr/bin/python
# Tempco plot demo app, requires matplotlib and Python 3.9
#  Saving plots have issue with Python 3.13
from colorama import just_fix_windows_console
just_fix_windows_console()
print ("\033[32;1mRevision 0.2 JAN.13.2025 | xDevs.com TCKIT plot tool")
print ("  (C) Illya Tsemenko | https://xdevs.com\033[39;0m")

import sys
import time
import numpy as np
import pandas as pd
import matplotlib
import math
#matplotlib.use('Agg')
import matplotlib.pyplot as plt
import csv
import scipy.ndimage.filters
import scipy.interpolate
import os
from urllib.request import pathname2url
from datetime import datetime
import matplotlib.dates as mdates
cwd = os.getcwd()

if (len(sys.argv) <= 1):
    fn = "mi6010cfm_rx_tg742_1_rs_xf742_50ma_tcr_apr2025.dsv"   # Hardcoded filename
else:
    fn = sys.argv[1]                               # Read data input filename from command line arg as alternative

fnout = cwd + "\\" + fn.split(".dsv")[0] + "_sorted.dsv"
print ("\033[44;33m-i- Input file: %s  \033[49;0m" % fn)
## ------------------ Configuration variables --------------------------------------------------- ##

start_sample = 500                          # Start sample for analysis
end_sample   = 256333                        # End sample for analysis
datespan = 8                                # Horizontal timescale resolution , hours/div

enable_detrending = True  # Apply data linear detrending if set to True
min_temp = 18                               # Minimum temperature for scale
max_temp = 28                               # Maximum temperature for scale
reftemp  = 23                               # Reference temperature for calculations, typically 23 C
tempcorr = 0.150
rs = 1.00003799                             # Reference RS resistance for ratio calculation (DCC method)
rs_unc = 0.6                                # Uncertainty for RS in ppms
ref_range1   = 10

# Reference value for relative calculations
ref_val1   = 1
ref_val2   = 1
ref_val3   = 1
ref_val4   = 1
ref_val5   = 10
ref_val6   = 10
ref_val7   = 10
ref_val8   = 10

# Correction offsets in ppms
ofs1 = 0
ofs2 = 0
ofs3 = 0
ofs4 = 0.0
ofs5 = 0.0
ofs6 = 0.0
ofs7 = 0.0
ofs8 = 0.0

stats_units = "Ω" # V Ω
dut1_name = "Fluke 742A-1, S/N 7790003"
dut2_name = "Branadic NTC, 10k box"
dut3_name = "MP1 NTC, YSI 44033"
dut4_name = "Shorted input test"
detector_ch1 = "MIL 6010B"
ref_str_ch1  = "xDevs.com 742A-1 %.8f %s ±%.2f µ%s/%s" % (rs, stats_units, rs_unc, stats_units, stats_units)

title_label = (r'TC Run sweep, 50mA 1:1 DCC, DUT: %s' % (dut1_name) )# µA 
axis4_x_label = "DUT Airbath temperature, °C"
axis2_x_label = "Timescale"

axis_y_label  = "DUT1 Relative error, µ%s/%s vs first 32 samples" % (stats_units, stats_units)
axis6_y_label = "DUT2 Relative error, µ%s/%s vs first 32 samples" % (stats_units, stats_units)
axis4_y_label = "DUT3 Relative error, µ%s/%s vs first 32 samples" % (stats_units, stats_units)
axis5_y_label = "DUT4 Relative error, µ%s/%s vs first 32 samples" % (stats_units, stats_units)
axis2_y_label = ("Sensor temperatures, °C" )
axis3_y_label = "Ambient temperature, °C"
axis3_yb_label = "Ambient humidity, %RH"
axis3_yc_label = "Ambient pressure, hPa"
axis9_y_label  = "DUT1 Deviation relative to temperature, µ%s/%s" % (stats_units, stats_units)
axis10_y_label = "DUT2 Deviation relative to temperature, µ%s/%s" % (stats_units, stats_units)
axis11_y_label = "DUT3 Deviation relative to temperature, µ%s/%s" % (stats_units, stats_units)
axis12_y_label = "DUT4 Deviation relative to temperature, µ%s/%s" % (stats_units, stats_units)

chart_name   = ("DUT1 TC sweep, µ%s/%s to nominal %g %s" % (stats_units, stats_units, round(ref_val1, 6), stats_units) )
chart6_name  = ("DUT2 TC sweep, µ%s/%s to nominal %g %s" % (stats_units, stats_units, round(ref_val2, 6), stats_units) )
chart4_name  = ("DUT3 TC sweep, µ%s/%s to nominal %g %s" % (stats_units, stats_units, round(ref_val3, 6), stats_units) )
chart5_name  = ("DUT4 TC sweep, µ%s/%s to nominal %g %s" % (stats_units, stats_units, round(ref_val4, 6), stats_units) )

chart9_name  = "DUT1 Temperature coefficient relative to airbath"
chart10_name = "DUT2 Temperature coefficient relative to airbath"
chart11_name = "DUT3 Temperature coefficient relative to airbath"
chart12_name = "DUT4 Temperature coefficient relative to airbath"

chart2_name = "Airbath parameters, DUT temperature"
chart3_name = "Environment conditions, temperature"
chart3b_name = "Environment conditions, humidity"
chart3c_name = "Environment conditions, pressure"

axis_y_max   = 1
axis_y_min   = -1
axis_y_step  = 0.1

axis_x_label = "Timescale, hours"
axis_x_max   = 10
axis_x_min   = 0
axis_x_step  = 4                         # Hours step

ax_y_resolution  = 0.1
ax6_y_resolution = 1
ax4_y_resolution = 1
ax5_y_resolution = 0.5

dataset_name1  =  "%s" % dut1_name
dataset_name2  =  "%s" % dut2_name
dataset_name3  =  "%s" % dut3_name
dataset_name4  =  "%s" % dut4_name
dataset_name5  =  "HP 34420A CH1"
dataset_name6  =  "HP 34420A CH2"
dataset_name7  =  "Keithley 2002+1801"
dataset_name8  =  "1σ error "
dataset_name9  =  "Datron 1281 20V, RESL8, FAST_OFF"
dataset_name10 =  "3458D (peak)"
dataset_name11 =  "8508A (peak)"
dataset_name12 =  "K182-M (peak)"

ax_legend_labels   = [dataset_name1, dataset_name1 + ', detrended']
ax6_legend_labels  = [dataset_name2, dataset_name2 + ', detrended']
ax4_legend_labels  = [dataset_name3, dataset_name3 + ', detrended']
ax5_legend_labels  = [dataset_name4, dataset_name4 + ', detrended']
ax9_legend_labels  = [dataset_name1 + ', detrended', dataset_name1, "Polyfit vs temperature"]
ax10_legend_labels = [dataset_name2 + ', detrended', dataset_name2, "Polyfit vs temperature"]
ax11_legend_labels = [dataset_name3 + ', detrended', dataset_name3, "Polyfit vs temperature"]
ax12_legend_labels = [dataset_name4 + ', detrended', dataset_name4, "Polyfit vs temperature"]
axtec_legend_labels = ['Programmed temperature, °C', 'DUT Sensor temperature, °C']

# Various matplotlib settings
img_resolution_w  = 3840  # Output plot resolution in pixels, full-size
img_resolution_h  = 2100  # Output plot resolution in pixels, full-size
img_resolution_ws = 2000  # Output plot resolution in pixels for thumbnail
img_resolution_hs = 1200  # Output plot resolution in pixels for thumbnail
#dateformat_parse  = "%d/%m/%Y-%H:%M:%S"
dateformat_parse  = "%d/%m/%Y-%H:%M:%S"
show_plot         = False

dark_theme      = False
if (dark_theme == True):
    dataset_color1  = '#4bd7fb'
    dataset_color2  = '#7e04e9'
    dataset_color3  = '#f01f69'
    dataset_color1  = '#ff9900'
    dataset_color1l = '#fbd7fb'
    dataset_color2l = '#fe04e9'
    dataset_color3l = '#d21ae8'
    dataset_color4l = '#ffc60c'
    dataset_color5l = '#f700f8'
    dataset_color6l = '#ffb6b5'
    dataset_color5  = '#0700f8'
    dataset_color6  = '#0fb6b5'
    dataset_color7  = '#f8c400'
    dataset_color8  = '#f749ff'
    dataset_color9  = '#0000ff'
    dataset_color10 = '#ff0000'
    dataset_color11 = '#ffcc22'
    dataset_color12 = '#0000ff'
    xdevs_bg_color  = '#000011'
    amb_color       = "#4444ff"
    rh_color        = "#44ff44"
    press_color     = "#ff4444"
    grid_color      = "#333333"
    poly_color      = "#ffffee"
    sv_temp_color   = "#999999"
    lbl_bg_color    = "#333333"  # Stats text box label background color
else:
    dataset_color1  = '#4bd7fb'
    dataset_color2  = '#7e04e9'
    dataset_color3  = '#f01f69'
    dataset_color4  = '#ff9900'
    dataset_color5  = '#0700f8'
    dataset_color6  = '#0fb6b5'
    dataset_color1l = '#fbd7fb'
    dataset_color2l = '#fe04e9'
    dataset_color3l = '#d21ae8'
    dataset_color4l = '#ffc60c'
    dataset_color5l = '#f700f8'
    dataset_color6l = '#ffb6b5'
    dataset_color7  = '#f8c400'
    dataset_color8  = '#f749ff'
    dataset_color9  = '#0000ff'
    dataset_color10 = '#ff0000'
    dataset_color11 = '#ffcc22'
    dataset_color12 = '#0000ff'
    xdevs_bg_color  = '#eeeeff'
    amb_color       = "#4444ee"
    rh_color        = "#44aa44"
    press_color     = "#aa4444"
    grid_color      = "#cccccc"
    poly_color      = "#000011"
    sv_temp_color   = "#111111"
    lbl_bg_color    = "#eeeeff"  # Stats text box label background color
## ----------------------------------------------------------------------------------------------- ##

# Resort data from input random DSV
#df = pd.read_csv(fn , skiprows=0, sep=';', index_col = 0)
#print ("-i- Read file into DF [A]")
#print ("\033[31;1m", df.head(8))
#df_out = df.sort_values(by=['duta'])
#print ("\033[31;1m", df_out.head(8))
#print (fnout)
#df_out.to_csv(fnout, sep=';')# header=None)

idxr = []
cntr = []

val1 = []
val2 = []
val3 = []
val4 = []
val5 = []
val6 = []
val7 = []
val8 = []
tmp116 = []
tmp117 = []
tmp119 = []
raw1 = []
raw2 = []
raw3 = []
raw4 = []
raw5 = []
raw6 = []
raw7 = []
raw8 = []

chdiff = []
gain1diff = []
gain2diff = []
gainhdiff = []
error1 = []
error2 = []
error3 = []
error4 = []
error5 = []
error6 = []
ambient = []
rh = []
pressure = []
count = 0
count_cut = 0
stamp = []

sv_temp = []
ysi_a = 1.032e-3
ysi_b = 3891
ysi_c = 1.58e-7

with open(fn) as csvfile:
    spamreader = csv.reader(csvfile, delimiter=';', quotechar='|')
    is_first = True
    for row in spamreader:
        if is_first:
            is_first = False
            continue
        count_cut = count_cut + 1
        if (start_sample < count_cut < end_sample):
            idxr.extend([row[0]]) #3458
            stamp.extend([datetime.strptime(row[0], dateformat_parse)])
            count = count + 1
            cntr.extend([count])
            val1.extend([ ofs5 + ( (rs * float(row[4]) / ref_val1) - 1) * 1e6 ])  # vbl5 : 34420-ch1
            #val1.extend([ ofs1 + ((float(row[1]) * 1e3 / ref_val1) - 1) * 1e6 ])   # vbl1 : 3458A
            val2.extend([ ofs2 + ((float(row[11]) / ref_val2) - 1) * 1e6 ])  # vbl2 : 3458B
            val3.extend([ ofs3 + ((float(row[14]) / ref_val3) - 1) * 1e6 ])  # vbl3 : 3459X
            val4.extend([ ofs4 + ((float(row[1])  / ref_val4) - 1) * 1e6 ])  # vbl4 : na
            val5.extend([ ofs5 + ((float(row[2])  / ref_val5) - 1) * 1e6 ])  # vbl5 : 34420-ch1
            val6.extend([ ofs6 + ((float(row[2])  / ref_val6) - 1) * 1e6 ])  # vbl6 : 34420-ch2
            val7.extend([ ofs7 + ((float(row[2])  / ref_val7) - 1) * 1e6 ])  # vbl7 : 
            val8.extend([ ofs8 + ((float(row[2])  / ref_val8) - 1) * 1e6 ])  # vbl8 :
            
            raw1.extend([ rs * float(row[4])  ])   # vbl1 : 3458A
            #raw1.extend([ float(row[1]) * 1e3  ])   # vbl1 : 3458A
            raw2.extend([ float(row[11]) ])  # vbl2 : 3458B
            raw3.extend([ float(row[14]) ])  # vbl3 : 3459X
            raw4.extend([ float(row[1]) ])  # vbl4 : na
            raw5.extend([ float(row[1]) ])  # vbl5 : 34420-ch1
            raw6.extend([ float(row[1]) ])  # vbl6 : 34420-ch2
            raw7.extend([ float(row[1]) ])  # vbl7 : 
            raw8.extend([ float(row[1]) ])  # vbl8 : 
            
            #tmp116.extend([ ((float(row[14]) - 10000.76) / 10000.76) * 1e3 + 23 ]) # ch1
            sv_temp.extend([ float(row[31]) ]) # ch1
            tmp117.extend([ float(row[29]) - tempcorr ])
            #tmp117.extend([(1 / ( ( math.log(float(row[11]) / 10000) / ysi_b + 1 / (23 + 273.15) ) ) ) - 273.15 ]) # ch1
            tmp119.extend([ float(row[28]) - tempcorr ]) # ch1
            
            #chdiff.extend([ (((float(row[10]) - 10000.76) / 10000.76) * 1e3 + 23) - (float(row[29]) - 0.4) ]) # ch-ch difference nV
            chdiff.extend([ (float(row[4]) - float(row[5]) ) * 1e6 ]) # ch-ch difference nV
            gain1diff.extend([ 1 ])
            gain2diff.extend([ 1 ])
            #gainhdiff.extend([ (( knv / float(row[6])) - 1) * 1e6 ])
            
            ambient.extend([float(row[25])]) # 3458
            rh.extend([float(row[26])]) # 2002
            pressure.extend([float(row[27])]) # 5720
        else:
            continue

print ("-i-\033[34;1m Analysis samples: %d\033[39;1m " % cntr[-1])
first_stamp = stamp[0]
last_stamp = stamp[-1]
diffstp = (stamp[-1] - stamp[0])
print ("\033[31;2m-i-", "First sample TS: ",first_stamp, "| Last sample TS: ", last_stamp, "\r\nData duration", diffstp, "\033[39;0m\r\n")

val1_detrended = []
val2_detrended = []
val3_detrended = []
val4_detrended = []
raw1_detrended = []
raw2_detrended = []
raw3_detrended = []
raw4_detrended = []

amb_max = np.amax(ambient)
amb_min = np.amin(ambient)
amb_span = amb_max - amb_min

press_max = np.amax(pressure)
press_min = np.amin(pressure)
press_span = press_max - press_min

tmp119_max = np.amax(tmp119)
tmp119_min = np.amin(tmp119)
tmp119_span = tmp119_max - tmp119_min

rh_max = np.amax(rh)
rh_min = np.amin(rh)
rh_span = rh_max - rh_min

print ("\033[32;1m-d- Median relative start for val1 = %.5f µ%s/%s" % (np.median(val1[:32]) , stats_units, stats_units) )
print ("\033[33;1m-d- Median relative start for val2 = %.5f µ%s/%s" % (np.median(val2[:32]) , stats_units, stats_units) )
print ("\033[34;1m-d- Median relative start for val3 = %.5f µ%s/%s" % (np.median(val3[:32]) , stats_units, stats_units) )
print ("\033[35;1m-d- Median relative start for val4 = %.5f µ%s/%s" % (np.median(val4[:32]) , stats_units, stats_units) )
print ("\033[32;1m-d- Median relative end for val1 = %.5f µ%s/%s" % (np.median(val1[-32:]), stats_units, stats_units) )
print ("\033[33;1m-d- Median relative end for val2 = %.5f µ%s/%s" % (np.median(val2[-32:]), stats_units, stats_units) )
print ("\033[34;1m-d- Median relative end for val3 = %.5f µ%s/%s" % (np.median(val3[-32:]), stats_units, stats_units) )
print ("\033[35;1m-d- Median relative end for val4 = %.5f µ%s/%s" % (np.median(val4[-32:]), stats_units, stats_units) )
print ("\033[31;2m-d- Median RAW for val1 = %.9f %s" % (np.median(raw1), stats_units) )
print ("\033[32;2m-d- Median RAW for val2 = %.9f %s" % (np.median(raw2), stats_units) )
print ("\033[33;2m-d- Median RAW for val3 = %.9f %s" % (np.median(raw3), stats_units) )
print ("\033[34;2m-d- Median RAW for val4 = %.9f %s" % (np.median(raw4), stats_units) )

# Calculate drift rates for relative values
start_ch1 = np.median(val1[:48])
end_ch1 = np.median(val1[-48:])
drift_ch1 = (start_ch1 - end_ch1)
if (enable_detrending == True):
    drift_rate_ch1 = (drift_ch1 / cntr[-1])
else:
    drift_rate_ch1 = 0

start_ch2 = np.median(val2[:48])
end_ch2 = np.median(val2[-48:])
drift_ch2 = (start_ch2 - end_ch2)
if (enable_detrending == True):
    drift_rate_ch2 = (drift_ch2 / cntr[-1])
else:
    drift_rate_ch2 = 0

start_ch3 = np.median(val3[:48])
end_ch3 = np.median(val3[-48:])
drift_ch3 = (start_ch3 - end_ch3)
if (enable_detrending == True):
    drift_rate_ch3 = (drift_ch3 / cntr[-1])
else:
    drift_rate_ch3 = 0

start_ch4 = np.median(val4[:48])
end_ch4   = np.median(val4[-48:])
drift_ch4 = (start_ch4 - end_ch4)
if (enable_detrending == True):
    drift_rate_ch4 = (drift_ch4 / cntr[-1])
else:
    drift_rate_ch4 = 0

# Calculate drift rates for RAW values
start_ch1r = np.median(raw1[:32])
end_ch1r = np.median(raw1[-32:])
drift_ch1r = (start_ch1r - end_ch1r)
if (enable_detrending == True):
    drift_rate_ch1r = (drift_ch1r / cntr[-1])
else:
    drift_rate_ch1r = 0

start_ch2r = np.median(raw2[:32])
end_ch2r = np.median(raw2[-32:])
drift_ch2r = (start_ch2r - end_ch2r)
if (enable_detrending == True):
    drift_rate_ch2r = (drift_ch2r / cntr[-1])
else:
    drift_rate_ch2r = 0

start_ch3r = np.median(raw3[:32])
end_ch3r   = np.median(raw3[-32:])
drift_ch3r = (start_ch3r - end_ch3r)
if (enable_detrending == True):
    drift_rate_ch3r = (drift_ch3r / cntr[-1])
else:
    drift_rate_ch3r = 0

start_ch4r = np.median(raw4[:32])
end_ch4r   = np.median(raw4[-32:])
drift_ch4r = (start_ch4r - end_ch4r)
if (enable_detrending == True):
    drift_rate_ch4r = (drift_ch4r / cntr[-1])
else:
    drift_rate_ch4r = 0

for ditr in range(0, cntr[-1]):
    val1_detrended.extend([ val1[ditr] + drift_rate_ch1 * ditr ])
    val2_detrended.extend([ val2[ditr] + drift_rate_ch2 * ditr ])
    val3_detrended.extend([ val3[ditr] + drift_rate_ch3 * ditr ])
    val4_detrended.extend([ val4[ditr] + drift_rate_ch4 * ditr ])
    raw1_detrended.extend([ raw1[ditr] + drift_rate_ch1r * ditr ])
    raw2_detrended.extend([ raw2[ditr] + drift_rate_ch2r * ditr ])
    raw3_detrended.extend([ raw3[ditr] + drift_rate_ch3r * ditr ])
    raw4_detrended.extend([ raw4[ditr] + drift_rate_ch4r * ditr ])

residual_ch1 = np.median(val1_detrended[:32]) - np.median(val1_detrended[-32:])
residual_ch2 = np.median(val2_detrended[:32]) - np.median(val2_detrended[-32:])
residual_ch3 = np.median(val3_detrended[:32]) - np.median(val3_detrended[-32:])
residual_ch4 = np.median(val4_detrended[:32]) - np.median(val4_detrended[-32:])

filtered1 = scipy.ndimage.gaussian_filter1d(val1, sigma=1)
filtered2 = scipy.ndimage.gaussian_filter1d(val2, sigma=1)
filtered3 = scipy.ndimage.gaussian_filter1d(val3, sigma=1)
filtered4 = scipy.ndimage.gaussian_filter1d(val4, sigma=1)
filtered5 = scipy.ndimage.gaussian_filter1d(val5, sigma=1)
filtered6 = scipy.ndimage.gaussian_filter1d(val6, sigma=1)

filtered1_dt = scipy.ndimage.gaussian_filter1d(val1_detrended, sigma=1)
filtered2_dt = scipy.ndimage.gaussian_filter1d(val2_detrended, sigma=1)
filtered3_dt = scipy.ndimage.gaussian_filter1d(val3_detrended, sigma=1)
filtered4_dt = scipy.ndimage.gaussian_filter1d(val4_detrended, sigma=1)

filtered1_min = np.min(filtered1)
filtered1_max = np.max(filtered1)
#ax.yaxis.set_ticks(np.arange(round(filtered5_min,0) - 0.1, round(filtered5_max,0) + 0.1, 0.1))
print (filtered1_min, filtered1_max)

# Calculate polyfits
#polych1 = np.polyfit(tmp119, raw1_detrended, 2) # Against ambient
polych1 = np.polyfit(tmp119, raw1_detrended, 2)
polych2 = np.polyfit(tmp119, raw2_detrended, 2)
polych3 = np.polyfit(tmp119, raw3_detrended, 2)
polych4 = np.polyfit(tmp119, raw4_detrended, 2)
#polych1_rel = np.polyfit(ambient, filtered1_dt, 2) # Use detrended data against ambient
polych1_rel = np.polyfit(tmp119, filtered1_dt, 2) # Use detrended data
polych2_rel = np.polyfit(tmp119, filtered2_dt, 2) # Use detrended data
polych3_rel = np.polyfit(tmp119, filtered3_dt, 2) # Use detrended data
polych4_rel = np.polyfit(tmp119, filtered4_dt, 2) # Use detrended data

c2   = polych1[0]
c1   = polych1[1]
gain = polych1[2]

c2b   = polych2[0]
c1b   = polych2[1]
gainb = polych2[2]

c2c   = polych3[0]
c1c   = polych3[1]
gainc = polych3[2]

c2d   = polych4[0]
c1d   = polych4[1]
gaind = polych4[2]

tvalue1 = (c2 * reftemp*reftemp + c1*reftemp + gain)
alpha1 = ((c1 + 2*reftemp*c2) / tvalue1*1e6 )
beta1 = (c2 / tvalue1*1e6 )
zerotctemp1 = (-alpha1 / 2 / beta1 + reftemp)

tvalue2 = (c2b * reftemp*reftemp + c1b*reftemp + gainb)
alpha2 = ((c1b + 2*reftemp*c2b) / tvalue2*1e6 )
beta2 = (c2b / tvalue2*1e6 )
zerotctemp2 = (-alpha2 / 2 / beta2 + reftemp)

tvalue3 = (c2c * reftemp*reftemp + c1c*reftemp + gainc)
alpha3 = ((c1c + 2*reftemp*c2c) / tvalue3*1e6 )
beta3 = (c2c / tvalue3*1e6 )
zerotctemp3 = (-alpha3 / 2 / beta3 + reftemp)

tvalue4 = (c2d * reftemp*reftemp + c1d*reftemp + gaind)
alpha4 = ((c1d + 2*reftemp*c2d) / tvalue4*1e6 )
beta4 = (c2d / tvalue4*1e6 )
zerotctemp4 = (-alpha4 / 2 / beta4 + reftemp)

print ("\033[33;2m\r\n-P- Polyfit ch1 coefficients (B²,A,V): ", polych1)
print ("\033[34;1m-P- Polyfit ch2 coefficients (B²,A,V): ", polych2)
print ("\033[35;1m-P- Polyfit ch3 coefficients (B²,A,V): ", polych3)
print ("\033[36;1m-P- Polyfit ch4 coefficients (B²,A,V): ", polych4)
print ("\033[33;2m-P- Value[1]: %.9f" % tvalue1, "Alpha 1: %.3f" % alpha1, "Beta 1: %.3f" % beta1, "Zero TC temp 1: %.3f C" % zerotctemp1)
print ("\033[34;1m-P- Value[2]: %.9f" % tvalue2, "Alpha 2: %.3f" % alpha2, "Beta 2: %.3f" % beta2, "Zero TC temp 2: %.3f C\033[39;0m" % zerotctemp2)
print ("\033[35;2m-P- Value[3]: %.9f" % tvalue3, "Alpha 3: %.3f" % alpha3, "Beta 3: %.3f" % beta3, "Zero TC temp 3: %.3f C" % zerotctemp3)
print ("\033[36;1m-P- Value[4]: %.9f" % tvalue4, "Alpha 4: %.3f" % alpha4, "Beta 4: %.3f" % beta4, "Zero TC temp 4: %.3f C\033[39;0m" % zerotctemp4)

print ("TMP119 temp channel data: ", tmp119_min, tmp119_max, tmp119_span)
chdiff_filt = scipy.ndimage.gaussian_filter1d(chdiff, sigma=1)

###------------------------------------------ PLOTTING Section code -------------------------###

if (dark_theme == True):
    plt.style.use('dark_background')
#fig, ((ax, ax6, ax4, ax5), (ax9, ax10, ax11, ax12), (axtec, axenv, axev2, axev3)) = plt.subplots(2,3)
fig, ((ax, ax9, axtec), ( axenv, axev2, axev3)) = plt.subplots(2,3)
fig.suptitle(title_label + (", %d samples" % count), size=18)

if (dark_theme == True):
    ax.patch.set_color('#000000')
else:
    ax.patch.set_color('#ffffff')
fig.patch.set_facecolor(xdevs_bg_color)
ax.set_clip_on(False)

fig.set_size_inches(img_resolution_w / 100,img_resolution_h / 100)
plt.tight_layout(rect=[0.03, 0.03, 0.982, 0.982], w_pad=0.1, h_pad=0.08)                                # Reduce margins around the graph
plt.subplots_adjust(hspace=0.12, wspace=0.1)

#axenvb = axenv.twinx()
# Optional pressure scale
#axenvc = axenv.twinx()
#axenvc.spines['right'].set_position(('axes', 1.125))   # Offset Y3 axis 

#ax6.plot(stamp, raw1, dataset_color1, label='Data %i' % (1), alpha=0.7, linewidth=0.2, marker="p", markersize=5, markerfacecolor=dataset_color1)
#ax6.plot(stamp, filtered2, dataset_color2, label='Data %i' % (1), alpha=0.7, linewidth=0.2, marker="p", markersize=5, markerfacecolor=dataset_color2)
#ax6.plot(stamp, filtered3, dataset_color3, label='Data %i' % (1), alpha=0.7, linewidth=0.2, marker="p", markersize=5, markerfacecolor=dataset_color3)
#ax.plot(stamp, filtered1   ,dataset_color1l, alpha=0.5, linewidth=1, marker=" ", markersize=5, markerfacecolor=dataset_color1l)
ax.plot(stamp, filtered1,dataset_color3 , alpha=0.8, linewidth=1, marker="1", markersize=5, markerfacecolor=dataset_color2)
#ax6.plot(stamp,filtered2   ,dataset_color2l, alpha=0.5, linewidth=1, marker=" ", markersize=5, markerfacecolor=dataset_color2l)
#ax6.plot(stamp,filtered2_dt,dataset_color2 , alpha=0.8, linewidth=1, marker="2", markersize=5, markerfacecolor=dataset_color2)
#ax4.plot(stamp,filtered3   ,dataset_color3l, alpha=0.5, linewidth=1, marker=" ", markersize=5, markerfacecolor=dataset_color3l)
#ax4.plot(stamp,filtered3_dt,dataset_color3 , alpha=0.8, linewidth=1, marker="3", markersize=5, markerfacecolor=dataset_color3)
#ax5.plot(stamp,filtered4   ,dataset_color4l, alpha=0.5, linewidth=1, marker=" ", markersize=5, markerfacecolor=dataset_color4l)
#ax5.plot(stamp,filtered4_dt,dataset_color4 , alpha=0.8, linewidth=1, marker="4", markersize=5, markerfacecolor=dataset_color4)

ax9.plot (tmp119, filtered1_dt, dataset_color3,  alpha=0.7, linewidth=1, marker="p", markersize=5, markerfacecolor=dataset_color3)
ax9.plot (tmp119, filtered1,    dataset_color3l, alpha=0.5, linewidth=1, marker=" ", markersize=2, markerfacecolor=dataset_color3l)
ax9.plot (tmp119, np.polyval(polych1_rel, tmp119), poly_color, linestyle='dotted', alpha=1, linewidth=2, marker=" ")
#ax9.plot (tmp119, filtered1_dt, dataset_color1,  alpha=0.7, linewidth=1, marker="p", markersize=5, markerfacecolor=dataset_color1)
#ax9.plot (tmp119, filtered1,    dataset_color1l, alpha=0.5, linewidth=1, marker=" ", markersize=2, markerfacecolor=dataset_color1l)
#ax9.plot (tmp119, np.polyval(polych1_rel, tmp119), poly_color, linestyle='dotted', alpha=1, linewidth=2, marker=" ")
#ax10.plot(tmp119, filtered2_dt, dataset_color2,  alpha=0.7, linewidth=1, marker="p", markersize=5, markerfacecolor=dataset_color2)
#ax10.plot(tmp119, filtered2,    dataset_color2l, alpha=0.5, linewidth=1, marker=" ", markersize=2, markerfacecolor=dataset_color2l)
#ax10.plot(tmp119, np.polyval(polych2_rel, tmp119), poly_color, linestyle='dotted', alpha=1, linewidth=2, marker=" ")
#ax11.plot(tmp119, filtered3_dt, dataset_color3,  alpha=0.7, linewidth=1, marker="p", markersize=5, markerfacecolor=dataset_color3)
#ax11.plot(tmp119, filtered3,    dataset_color3l, alpha=0.5, linewidth=1, marker=" ", markersize=2, markerfacecolor=dataset_color3l)
#ax11.plot(tmp119, np.polyval(polych3_rel, tmp119), poly_color, linestyle='dotted', alpha=1, linewidth=2, marker=" ")
#ax12.plot(tmp119, filtered4_dt, dataset_color4,  alpha=0.7, linewidth=1, marker="p", markersize=5, markerfacecolor=dataset_color4)
#ax12.plot(tmp119, filtered4,    dataset_color4l, alpha=0.5, linewidth=1, marker=" ", markersize=2, markerfacecolor=dataset_color4l)
#ax12.plot(tmp119, np.polyval(polych4_rel, tmp119), poly_color, linestyle='dotted', alpha=1, linewidth=2, marker=" ")

axtec.plot(stamp, tmp119, "#50e991", label='Data %i' % (4), alpha=0.8, linewidth=2, marker="+", markersize=2, markerfacecolor="#fbe04e")
axtec.plot(stamp, tmp117, "#fd7f6f", alpha=1, linewidth=2, marker=".", markersize=1, markerfacecolor="#e50049")
axtec.plot(stamp, sv_temp, sv_temp_color, alpha=1, linewidth=1, linestyle='dashed', marker=" ", markersize=1, markerfacecolor="#000000")

axenv.plot(stamp, ambient, amb_color, label='Ambient T, °C', alpha=1, linewidth=1, marker="^", markersize=1, markerfacecolor=amb_color)
axev2.plot(stamp, rh, rh_color, label='Humidity, %%RH', alpha=1, linewidth=1, marker="s", markersize=1, markerfacecolor=rh_color)
axev3.plot(stamp, pressure, press_color, label='Pressure, hPa', alpha=1, linewidth=1, marker="o", markersize=1, markerfacecolor=press_color)

axenv.axis([axis_x_min, axis_x_max, amb_min * 0.99, amb_max * 1.01])
axev2.axis([axis_x_min, axis_x_max, rh_min * 0.99, rh_max * 1.01])
axev3.axis([axis_x_min, axis_x_max, press_min * 0.99, press_max * 1.01])

ax9.xaxis.set_ticks(np.arange(round(tmp119_min,0) - 1, round(tmp119_max,0) + 1, 0.5))
#ax10.xaxis.set_ticks(np.arange(round(tmp119_min,0) - 1, round(tmp119_max,0) + 1, 1))
#ax11.xaxis.set_ticks(np.arange(round(tmp119_min,0) - 1, round(tmp119_max,0) + 1, 1))
#ax12.xaxis.set_ticks(np.arange(round(tmp119_min,0) - 1, round(tmp119_max,0) + 1, 1))

ax.text(0.02, 0.03, ("Residual drift: %.3f µ%s/%s" % (residual_ch1, stats_units, stats_units)), horizontalalignment='left', verticalalignment='center', transform = ax.transAxes, bbox=dict(facecolor=lbl_bg_color, alpha=0.55))
ax.text(0.4, 0.03, ("Detector: %s" % detector_ch1), horizontalalignment='left', verticalalignment='center', transform = ax.transAxes, bbox=dict(facecolor=lbl_bg_color, alpha=0.55))
ax.text(0.35, 0.09, ("REF: %s" % (ref_str_ch1)), horizontalalignment='left', verticalalignment='center', transform = ax.transAxes, bbox=dict(facecolor=lbl_bg_color, alpha=0.55))

#ax6.text(0.02, 0.03,("Residual drift: %.3f µ%s/%s" % (residual_ch2, stats_units, stats_units)), horizontalalignment='left', verticalalignment='center', transform = ax6.transAxes, bbox=dict(facecolor=lbl_bg_color, alpha=0.55))
#ax4.text(0.02, 0.03,("Residual drift: %.3f µ%s/%s" % (residual_ch3, stats_units, stats_units)), horizontalalignment='left', verticalalignment='center', transform = ax4.transAxes, bbox=dict(facecolor=lbl_bg_color, alpha=0.55))
#ax5.text(0.02, 0.03,("Residual drift: %.3f µ%s/%s" % (residual_ch4, stats_units, stats_units)), horizontalalignment='left', verticalalignment='center', transform = ax5.transAxes, bbox=dict(facecolor=lbl_bg_color, alpha=0.55))

axis45_x_pos = 0.2

#vert_place_lbls = [0.23, 0.16, 0.10, 0.02]
vert_place_lbls = [0.20, 0.14, 0.08, 0.02]
ax9.text(axis45_x_pos , vert_place_lbls[0],("DUT1₂₃: %9.9g %s" % (tvalue1, stats_units)), fontsize=12,  horizontalalignment='left', verticalalignment='center', transform = ax9.transAxes,  bbox=dict(facecolor=lbl_bg_color, alpha=0.55))
ax9.text(axis45_x_pos , vert_place_lbls[1],("α: %.3f µ%s/%s/K" % (alpha1, stats_units, stats_units)),   horizontalalignment='left', verticalalignment='center', transform = ax9.transAxes,  bbox=dict(facecolor=lbl_bg_color, alpha=0.55))
ax9.text(axis45_x_pos , vert_place_lbls[2],("β : %.3f µ%s/%s/K²" % (beta1, stats_units, stats_units)), horizontalalignment='left', verticalalignment='center', transform = ax9.transAxes,  bbox=dict(facecolor=lbl_bg_color, alpha=0.55))
ax9.text(axis45_x_pos , vert_place_lbls[3],("Zero TC: %.1f °C" % (zerotctemp1)),                        horizontalalignment='left', verticalalignment='bottom', transform = ax9.transAxes,  bbox=dict(facecolor=lbl_bg_color, alpha=0.55))
#ax10.text(axis45_x_pos, vert_place_lbls[0],("DUT2₂₃: %9.9g %s" % (tvalue2, stats_units)), fontsize=12,  horizontalalignment='left', verticalalignment='center', transform = ax10.transAxes, bbox=dict(facecolor=lbl_bg_color, alpha=0.55))
#ax10.text(axis45_x_pos, vert_place_lbls[1],("α: %.3f µ%s/%s/K" % (alpha2, stats_units, stats_units)),   horizontalalignment='left', verticalalignment='center', transform = ax10.transAxes, bbox=dict(facecolor=lbl_bg_color, alpha=0.55))
#ax10.text(axis45_x_pos, vert_place_lbls[2],("β : %.3f µ%s/%s/K²" % (beta2, stats_units, stats_units)), horizontalalignment='left', verticalalignment='center', transform = ax10.transAxes, bbox=dict(facecolor=lbl_bg_color, alpha=0.55))
#ax10.text(axis45_x_pos, vert_place_lbls[3],("Zero TC: %.1f °C" % (zerotctemp2)),                        horizontalalignment='left', verticalalignment='bottom', transform = ax10.transAxes, bbox=dict(facecolor=lbl_bg_color, alpha=0.55))
#ax11.text(axis45_x_pos, vert_place_lbls[0],("DUT3₂₃: %9.9g %s" % (tvalue3, stats_units)), fontsize=12,  horizontalalignment='left', verticalalignment='center', transform = ax11.transAxes, bbox=dict(facecolor=lbl_bg_color, alpha=0.55))
#ax11.text(axis45_x_pos, vert_place_lbls[1],("α: %.3f µ%s/%s/K" % (alpha3, stats_units, stats_units)),   horizontalalignment='left', verticalalignment='center', transform = ax11.transAxes, bbox=dict(facecolor=lbl_bg_color, alpha=0.55))
#ax11.text(axis45_x_pos, vert_place_lbls[2],("β : %.3f µ%s/%s/K²" % (beta3, stats_units, stats_units)), horizontalalignment='left', verticalalignment='center', transform = ax11.transAxes, bbox=dict(facecolor=lbl_bg_color, alpha=0.55))
#ax11.text(axis45_x_pos, vert_place_lbls[3],("Zero TC: %.1f °C" % (zerotctemp3)),                        horizontalalignment='left', verticalalignment='bottom', transform = ax11.transAxes, bbox=dict(facecolor=lbl_bg_color, alpha=0.55))
#ax12.text(axis45_x_pos, vert_place_lbls[0],("DUT4₂₃: %9.9g %s" % (tvalue4, stats_units)), fontsize=12,  horizontalalignment='left', verticalalignment='center', transform = ax12.transAxes, bbox=dict(facecolor=lbl_bg_color, alpha=0.55))
#ax12.text(axis45_x_pos, vert_place_lbls[1],("α: %.3f µ%s/%s/K" % (alpha4, stats_units, stats_units)),   horizontalalignment='left', verticalalignment='center', transform = ax12.transAxes, bbox=dict(facecolor=lbl_bg_color, alpha=0.55))
#ax12.text(axis45_x_pos, vert_place_lbls[2],("β : %.3f µ%s/%s/K²" % (beta4, stats_units, stats_units)), horizontalalignment='left', verticalalignment='center', transform = ax12.transAxes, bbox=dict(facecolor=lbl_bg_color, alpha=0.55))
#ax12.text(axis45_x_pos, vert_place_lbls[3],("Zero TC: %.1f °C" % (zerotctemp4)),                        horizontalalignment='left', verticalalignment='bottom', transform = ax12.transAxes, bbox=dict(facecolor=lbl_bg_color, alpha=0.55))

if (amb_span < 0.25):
    temp_box_color = "#cff800"
elif (0.25 >= amb_span < 0.5):
    temp_box_color = "#ffbf65"
elif (0.5 >= amb_span < 1.0):
    temp_box_color = "#ffec59"
else:
    temp_box_color = "#ff5c77"
    
if (amb_span < 15):
    rh_box_color = "#00f8ee"
elif (15 >= amb_span < 60):
    rh_box_color = "#2fff25"
elif (60 >= amb_span):
    rh_box_color = "#ff5c00"

axenv.text(0.02, 0.05,("T Max: %.2f °C, Min %.2f °C " % (amb_max , amb_min)), horizontalalignment='left', verticalalignment='center', transform = axenv.transAxes, bbox=dict(facecolor=temp_box_color, alpha=0.65))
axenv.text(0.6 , 0.05, ("T span +/-%.2f °C peak" % (amb_span/2)), horizontalalignment='left', verticalalignment='center', transform = axenv.transAxes, bbox=dict(facecolor=temp_box_color, alpha=0.65))
axev2.text(0.02, 0.05,("H Max: %.2f %%RH, Min %.2f %%RH " % (rh_max , rh_min)), horizontalalignment='left', verticalalignment='center', transform = axev2.transAxes, bbox=dict(facecolor=rh_box_color, alpha=0.65))
axev2.text(0.6 , 0.05, ("H span +/-%.2f %%RH peak" % (rh_span/2)), horizontalalignment='left', verticalalignment='center', transform = axev2.transAxes, bbox=dict(facecolor=rh_box_color, alpha=0.65))
axev3.text(0.02, 0.05,("P Max: %.1f hPa, Min %.1f hPa" % (press_max , press_min)), horizontalalignment='left', verticalalignment='center', transform = axev3.transAxes, bbox=dict(facecolor=rh_box_color, alpha=0.65))
axev3.text(0.6 , 0.05, ("P span +/-%.1f hPa peak" % (rh_span/2)), horizontalalignment='left', verticalalignment='center', transform = axev3.transAxes, bbox=dict(facecolor=rh_box_color, alpha=0.65))

start, end = ax.get_ylim()

#ax.yaxis.set_ticks(np.arange(start, end, axis_y_step))
#ax.yaxis.set_ticks (np.arange(start, end, axis_y_step * 2))
#axtec.yaxis.set_ticks(np.arange(start2, end2, axis_y_step * 2))

startx, endx = ax.get_xlim()

#ax.xaxis.set_ticks (np.arange(first_stamp, last_stamp, axis_x_step))
ax.xaxis.set_major_locator(mdates.HourLocator(byhour=range(0, 24, datespan)))
ax.xaxis.set_major_formatter(mdates.DateFormatter('%H h'))
ax.set_xlim(first_stamp, last_stamp)
ax.yaxis.set_major_locator(plt.MultipleLocator(ax_y_resolution))

#ax6.xaxis.set_major_locator(mdates.HourLocator(byhour=range(0, 24, datespan)))
#ax6.xaxis.set_major_formatter(mdates.DateFormatter('%H h'))
#ax6.set_xlim(first_stamp, last_stamp)
#ax6.yaxis.set_major_locator(plt.MultipleLocator(ax6_y_resolution))
#
#ax4.xaxis.set_major_locator(mdates.HourLocator(byhour=range(0, 24, datespan)))
#ax4.xaxis.set_major_formatter(mdates.DateFormatter('%H h'))
#ax4.set_xlim(first_stamp, last_stamp)
#ax4.yaxis.set_major_locator(plt.MultipleLocator(ax4_y_resolution))
#
#ax5.xaxis.set_major_locator(mdates.HourLocator(byhour=range(0, 24, datespan)))
#ax5.xaxis.set_major_formatter(mdates.DateFormatter('%H h'))
#ax5.set_xlim(first_stamp, last_stamp)
#ax5.yaxis.set_major_locator(plt.MultipleLocator(ax5_y_resolution))

axtec.axis([axis_x_min, axis_x_max, min_temp - 0.5, max_temp + 0.5])
#axtec.yaxis.set_ticks(np.arange(round(tmp119_min,0) - 1, round(tmp119_max,0) + 1, 0.5))
#axtec.axis([axis_x_min, axis_x_max, start2, end2])
axtec.yaxis.set_major_locator(plt.MultipleLocator(1.0))
axtec.set_ylim(min_temp - 0.5, max_temp + 0.5)
axtec.xaxis.set_major_locator(mdates.HourLocator(byhour=range(0, 24, datespan)))
axtec.xaxis.set_major_formatter(mdates.DateFormatter('%H h'))
axtec.set_xlim(first_stamp, last_stamp)
#axtec.set_ylim(min_temp, max_temp)

axenv.xaxis.set_major_locator(mdates.HourLocator(byhour=range(0, 24, datespan)))
axenv.xaxis.set_major_formatter(mdates.DateFormatter('%H h'))

axev2.xaxis.set_major_locator(mdates.HourLocator(byhour=range(0, 24, datespan)))
axev2.xaxis.set_major_formatter(mdates.DateFormatter('%H h'))

axev3.xaxis.set_major_locator(mdates.HourLocator(byhour=range(0, 24, datespan)))
axev3.xaxis.set_major_formatter(mdates.DateFormatter('%H h'))

axenv.set_xlim(first_stamp, last_stamp)
axev2.set_xlim(first_stamp, last_stamp)
axev3.set_xlim(first_stamp, last_stamp)

axenv.tick_params(axis='y', colors=amb_color)
axev2.tick_params(axis='y', colors=rh_color)
axev3.tick_params(axis='y', colors=press_color)

#ax5.xaxis.set_major_locator(mdates.HourLocator(byhour=range(0, 24, datespan)))
#ax5.xaxis.set_major_formatter(mdates.DateFormatter('%H h'))
#ax5.set_xlim(first_stamp, last_stamp)

ax9.set_xlim(min_temp - 0.5, max_temp + 0.5)
ax9.xaxis.set_major_locator(plt.MultipleLocator(1.0))
ax9.yaxis.set_major_locator(plt.MultipleLocator(ax_y_resolution))

#ax10.set_xlim(min_temp - 0.5, max_temp + 0.5)
#ax10.xaxis.set_major_locator(plt.MultipleLocator(1.0))
#ax10.yaxis.set_major_locator(plt.MultipleLocator(ax6_y_resolution))
#
#ax11.set_xlim(min_temp - 0.5, max_temp + 0.5)
#ax11.xaxis.set_major_locator(plt.MultipleLocator(1.0))
#ax11.yaxis.set_major_locator(plt.MultipleLocator(ax4_y_resolution))
#
#ax12.set_xlim(min_temp - 0.5, max_temp + 0.5)
#ax12.xaxis.set_major_locator(plt.MultipleLocator(1.0))
#ax12.yaxis.set_major_locator(plt.MultipleLocator(ax5_y_resolution))

#plt.xticks(pd.date_range(start="2023-01-01", end="2023-12-31", freq="M"))
#axtec.xaxis.set_ticks(np.arange(0, count, axis_x_step))
#axenv.xaxis.set_ticks(np.arange(0, count, axis_x_step))
#ax5.xaxis.set_ticks(np.arange(0, count, axis_x_step))

ax.set (xlabel=axis_x_label, ylabel=axis_y_label,   title='%s' % (chart_name))
#ax6.set(xlabel=axis_x_label, ylabel=axis6_y_label,   title='%s' % (chart6_name))
#ax4.set(xlabel=axis_x_label, ylabel=axis4_y_label,   title='%s' % (chart4_name))
#ax5.set(xlabel=axis_x_label, ylabel=axis5_y_label,   title='%s' % (chart5_name))

ax9.set (xlabel=axis4_x_label, ylabel=axis9_y_label,   title='%s' % (chart9_name))
#ax10.set(xlabel=axis4_x_label, ylabel=axis10_y_label,  title='%s' % (chart10_name))
#ax11.set(xlabel=axis4_x_label, ylabel=axis11_y_label,  title='%s' % (chart11_name))
#ax12.set(xlabel=axis4_x_label, ylabel=axis12_y_label,  title='%s' % (chart12_name))

axtec.set(xlabel=axis2_x_label,ylabel=axis2_y_label,  title='%s' % (chart2_name))
axenv.set(xlabel=axis_x_label, title='%s' % (chart3_name))
axev2.set(xlabel=axis_x_label, title='%s' % (chart3b_name))
axev3.set(xlabel=axis_x_label, title='%s' % (chart3c_name))

axenv.set_ylabel(ylabel=axis3_y_label, color=amb_color)
axev2.set_ylabel(ylabel=axis3_yb_label, color=rh_color)
axev3.set_ylabel(ylabel=axis3_yc_label, color=press_color)

#axtec.xaxis.set_ticks(np.arange(0, count, 2))

#start4, end4 = ax4.get_ylim()
#ax4.yaxis.set_ticks(np.arange(start4, end4, axis_y_step))
#ax4.xaxis.set_ticks(np.arange(0, count, 20))

# Plot grids on each of the subplots
ax.grid (color=grid_color)
axtec.grid(color=grid_color)
axenv.grid(color=grid_color)
axev2.grid(color=grid_color)
axev3.grid(color=grid_color)
#ax4.grid(color=grid_color)
#ax5.grid(color=grid_color)
ax9.grid(color=grid_color)

png_fn_large = fn.split(".dsv")[0] + ".png"
png_fn_small = fn.split(".dsv")[0] + "_1.png"

xcsfont = {'fontname':'HandelGothic BT'}
xbsfont = {'fontname':'Cambria'}

axev3.text(1.025, 0.5, '© xDevs.com | https://github.com/tin-/tckit', **xcsfont, rotation=90, fontsize=14, horizontalalignment='left', verticalalignment='center', transform = axev3.transAxes)
ax9.text(1.025, 0.5, "Start: %s        End: %s" % (first_stamp, last_stamp), **xbsfont, rotation=90, fontsize=16, horizontalalignment='left', verticalalignment  ='center', transform = ax9.transAxes)
axtec.text(1.025, 0.5, ("Duration: %s" % (last_stamp - first_stamp) ), **xbsfont, rotation=90, fontsize=16, horizontalalignment='left', verticalalignment  ='center', transform = axtec.transAxes)

ax.legend(ax_legend_labels     , loc='upper center')
#ax6.legend(ax6_legend_labels   , loc='upper center')
#ax4.legend(ax4_legend_labels   , loc='upper center')
#ax5.legend(ax5_legend_labels   , loc='upper center')
ax9.legend (ax9_legend_labels  , loc='upper right')
#ax10.legend(ax10_legend_labels , loc='upper right')
#ax11.legend(ax11_legend_labels , loc='upper right')
#ax12.legend(ax12_legend_labels , loc='upper right')
axtec.legend(axtec_legend_labels)
axenv.legend(["Ambient temperature, °C"], loc='upper right')
axev2.legend(["Ambient humidity, %%RH"], loc='upper left')
axev3.legend(["Ambient pressure, hPa"], loc='upper left')

if (show_plot == True):
    plt.show()


fig.savefig(png_fn_large, facecolor=fig.get_facecolor(), transparent=False)
print ("\033[44;33m-i- Saved PNG large: %s  \033[49;0m" % png_fn_large)
#axenv.yaxis.set_ticks (np.arange(start3, end3, axis_y_step * 2))
plt.tight_layout(rect=[0.02, 0.02, 0.99, 0.96], w_pad=0.1, h_pad=0.1)                                # Reduce margins around the graph
plt.subplots_adjust(hspace=0.15, wspace=0.15)

figs = fig
figs.set_size_inches(img_resolution_ws / 100, img_resolution_hs / 100)
figs.savefig(png_fn_small, facecolor=figs.get_facecolor(), transparent=False)
print ("\033[44;33m-i- Saved PNG small: %s  \033[49;0m" % png_fn_small)
print ("\033[49;32m-i- Job done  \033[49;39m")

